%{ // -*- C++ -*-

// (f)lex grammar for Esterel

#include <common/lstring.h>
#include "parse.h"

int yylineno = 1;

extern YYLTYPE yylloc;

#define STRINGTABLESIZE 701

// stringtable strings(STRINGTABLESIZE);	// The string table

// The first thing yyparse() does upon entry:

#define YY_USER_ACTION \
yylloc.first_line = yylineno;

// always assume this is the last file

#define yywrap() 1

%}

LETTER		[A-Za-z]
DIGIT		[0-9]
INTEGER		[0-9]+
ID		{LETTER}({LETTER}|{DIGIT}|_)*
WHITESPACE	[ \t\f]+
SINGLECHAR	[+\-*/<>.,=;:()\]\[?]
STRINGCONSTANT  ["][^"]*["]

%%

%[^\n]*\n	{ yylineno++; }
\n		{ yylineno++; }

{WHITESPACE}	{ }

"abort"		{ return(ABORT           ); }
"and"		{ return(AND             ); }
"await"		{ return(AWAIT           ); }
"call"		{ return(CALL            ); }
"case"		{ return(CASE            ); }
"combine"      	{ return(COMBINE         ); }
"constant"	{ return(CONSTANT        ); }
"copymodule"	{ return(COPYMODULE      ); }
"do"		{ return(DO              ); }
"each"		{ return(EACH            ); }
"elsif"		{ return(ELSIF           ); }
"else"		{ return(ELSE            ); }
"emit"		{ return(EMIT            ); }
"end"		{ return(END             ); }
"every"		{ return(EVERY           ); }
"exit"		{ return(EXIT            ); }
"false"		{ return(FALSE           ); }
"function"	{ return(FUNCTION        ); }
"halt"		{ return(HALT            ); }
"handle"	{ return(HANDLE          ); }
"if"		{ return(IF              ); }
"immediate"	{ return(IMMEDIATE       ); }
"in"		{ return(IN              ); }
"input"		{ return(INPUT           ); }
"inputoutput"	{ return(INPUTOUTPUT     ); }
"loop"		{ return(LOOP            ); }
"mod"		{ return(MOD             ); }
"module"	{ return(MODULE          ); }
"not"		{ return(NOT             ); }
"nothing"	{ return(NOTHING         ); }
"or"		{ return(OR              ); }
"output"	{ return(OUTPUT          ); }
"pause"		{ return(PAUSE           ); }
"positive"	{ return(POSITIVE        ); }
"pre"		{ return(PRE             ); }
"present"	{ return(PRESENT         ); }
"procedure"	{ return(PROCEDURE       ); }
"relation"	{ return(RELATION        ); }
"repeat"	{ return(REPEAT          ); }
"run"		{ return(RUN	         ); }
"return"	{ return(RETURN          ); }
"sensor"	{ return(SENSOR          ); }
"signal"	{ return(SIGNAL          ); }
"suspend"       { return(SUSPEND         ); }
"sustain"	{ return(SUSTAIN	 ); }
"then"		{ return(THEN            ); }
"timeout"	{ return(TIMEOUT         ); }
"times"		{ return(TIMES           ); }
"trap"		{ return(TRAP            ); }
"true"		{ return(TRUE            ); }
"type"		{ return(TYPE            ); }
"upto"		{ return(UPTO            ); }
"var"		{ return(VAR             ); }
"weak"		{ return(WEAK            ); }
"watching"	{ return(WATCHING        ); }
"when"		{ return(WHEN            ); }
"with"		{ return(WITH            ); }

"integer"	{ return(INTEGER	 ); }
"boolean"	{ return(BOOLEAN	 ); }

":="		{ return(COLEQ		 ); }
"=>"            { return(IMPLIES         ); }

"<="		{ return(LE		 ); }
">="		{ return(GE		 ); }
"<>"		{ return(NE		 ); }

"??"		{ return(TWOQUEST	 ); }

"||"		{ return(PARALLEL	 ); }

{SINGLECHAR}	{ return(yytext[0]	 ); }

{INTEGER}	{ yylval.i = atoi(yytext); return(INTLIT); }
{STRINGCONSTANT} { return(STRINGCONST); }

  /*
   * This relies on the LString class behaving like a compiler's string table,
   * i.e., once a string is entered, it never leaves memory.
   */

{ID}		{ LString ls = yytext; yylval.s = ls.c_str(); return(ID); }

%%
