#ifndef STATEMENT_DISMANTLERS_H
#define STATEMENT_DISMANTLERS_H

#include <basicnodes/basic.h>
#include <suifpasses/suifpasses.h>
#include <esuifnodes/esuif.h>
#include <suifkernel/module_subsystem.h>
#include <suifkernel/suif_env.h>
#include <suifkernel/command_line_parsing.h>
#include <suifnodes/suif.h>
#include <typebuilder/type_builder.h>
#include <transforms/procedure_walker_utilities.h>

extern void move_annotes(AnnotableObject *, AnnotableObject *);

template <class S> class Dismantler : public ProcedureWalker {
  SuifEnv *env;
public:
  Dismantler(SuifEnv *_env, ProcedureDefinition *pd) :
    ProcedureWalker(_env, pd, S::get_class_name()),
    env(_env) {}
  Walker::ApplyStatus operator () (SuifObject *o) {
    S *s = to<S>(o);
    AnnotableObject *result = dismantle(s);
    o->get_parent()->replace(o, result);
    move_annotes(s, result);
    set_address(result);
    return Walker::Replaced;
  }
  AnnotableObject *dismantle(S *);
};

template <class S> class ObjectWalker : public ProcedureWalker {
  SuifEnv *env;
public:
  ObjectWalker(SuifEnv *_env, ProcedureDefinition *pd) :
    ProcedureWalker(_env, pd, S::get_class_name()), env(_env) {}
  Walker::ApplyStatus operator () (SuifObject *o) {
    S *s = to<S>(o);
    walk(s);
    return Walker::Continue;
  }
  void walk(S *);
};

class ParallelTrapExitModule : public PipelinablePass {
  String _description;
  static QualifiedType *_integer_type;
public:
  ParallelTrapExitModule(SuifEnv *env, const LString &name,
			 const String &description)
    : PipelinablePass(env, name), _description(description) {}

  virtual ~ParallelTrapExitModule() {}

  void initialize() {
    PipelinablePass::initialize();
    _command_line->set_description(_description);
  }

  virtual void do_procedure_definition(ProcedureDefinition *);  

  Module *clone() const { return(Module*)this; }

  static QualifiedType *get_integer_type() { return _integer_type; }
};

class ThreadCreator : public Dismantler<ParallelStatementList> {
  SuifEnv *env;
public:
  ThreadCreator(SuifEnv *_env, ProcedureDefinition *pd) :
    Dismantler<ParallelStatementList>(_env, pd), env(_env) {}
  Statement *dismantle(ParallelStatementList *);
};

class ExitLevelVariableWalker : public ProcedureWalker {
public:
  ExitLevelVariableWalker(SuifEnv *env, ProcedureDefinition *pd) :
    ProcedureWalker(env, pd, ParallelStatementList::get_class_name()) {}
  Walker::ApplyStatus operator () (SuifObject *); 
  static void add_exit_level_variable(ParallelStatementList *);
};

/*
class FixProcDefs : public PipelinablePass {
  String _description;
public:
  FixProcDefs(SuifEnv *env, const LString &name, const String &description)
    : PipelinablePass(env, name), _description(description) {}

  virtual ~FixProcDefs() {}

  void initialize() {
    PipelinablePass::initialize();	
    _command_line->set_description(_description);
  }

  virtual void do_procedure_definition(ProcedureDefinition *);

  Module *clone() const { return (Module*)this; }
};
*/

class BuildCInterface : public PipelinablePass {
  String _description;
public:
  BuildCInterface(SuifEnv *env, const LString &name, const String &description)
    : PipelinablePass(env, name), _description(description) {}

  virtual ~BuildCInterface() {}

  void initialize() {
    PipelinablePass::initialize();	
    _command_line->set_description(_description);
  }

  virtual void do_procedure_definition(ProcedureDefinition *);

  Module *clone() const { return (Module*)this; }
};

#if 0
class ParallelTrapStatementDismantler : public PipelinablePass {
  String _description;
public:
  ParallelTrapStatementDismantler(SuifEnv *env, const LString& name,
				  const String &description)
    : PipelinablePass(env, name), _description(description) {}

  void initialize() {
    PipelinablePass::initialize();
    _command_line->set_description(_description);
  }

  virtual void do_procedure_definition(ProcedureDefinition *);

  struct Context {
    int depth;
  };

  void walk_statement(Statement *, const Context &);

  Module *clone() const { return (Module*)this; }
};
#endif

#endif
